'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createDiscovery;

var _dgram = require('dgram');

var _dgram2 = _interopRequireDefault(_dgram);

var _constants = require('../constants');

var _rx = require('rx');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var AUTODISCOVERY_PAYLOAD = '{"cmd":"autodiscover"}';

function createDiscovery() {
  return _rx.Observable.create(function (observer) {
    var client = _dgram2.default.createSocket('udp4');

    client.bind(undefined, undefined, function () {
      client.setBroadcast(true);

      var message = new Buffer(AUTODISCOVERY_PAYLOAD);

      client.send(message, 0, message.length, _constants.FALLOUT_UDP_PORT, '255.255.255.255', function (err) {
        if (err) {
          observer.onError(err);
        }
      });

      client.on('message', function (msg, rinfo) {
        try {
          var data = Object.assign(JSON.parse(msg.toString()), {
            info: rinfo
          });
          observer.onNext(data);
        } catch (err) {
          observer.onError(err);
        }
      });

      client.on('error', function (err) {
        observer.onError(err);
      });

      client.on('close', function () {
        observer.onCompleted();
      });
    });

    return function () {
      client.close();
    };
  });
}