'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = parseBinaryDatabase;
function parseBinaryDatabase(buf) {
  var offset = 0;
  var bundle = {};

  while (offset < buf.length) {
    var type = buf.readInt8(offset);
    var id = buf.readInt32LE(offset + 1);
    var length = 5;

    switch (type) {
      case 0:
        {
          var data = buf.readUInt8(offset + length);
          bundle[id] = data === 1 ? true : false;
          length += 1;
          break;
        }

      case 1:
        {
          var data = buf.readInt8(offset + length);
          bundle[id] = data;
          length += 1;
          break;
        }

      case 2:
        {
          var data = buf.readUInt8(offset + length);
          bundle[id] = data;
          length += 1;
          break;
        }

      case 3:
        {
          var data = buf.readInt32LE(offset + length);
          bundle[id] = data;
          length += 4;
          break;
        }

      case 4:
        {
          var data = buf.readUInt32LE(offset + length);
          bundle[id] = data;
          length += 4;
          break;
        }

      case 5:
        {
          var data = buf.readFloatLE(offset + length);
          bundle[id] = data;
          length += 4;
          break;
        }

      case 6:
        {
          // Read zero terminated string
          var _size = 0;
          do {
            _size++;
            if (offset + length + _size > buf.length) {
              throw 'Can\'t find termination of string. Data incomplete?';
            }
          } while (buf[offset + length + _size - 1] != 0x00);

          var data = buf.slice(offset + length, offset + length + _size - 1).toString('utf8');
          bundle[id] = data;
          length += _size;
          break;
        }

      case 7:
        {
          // Parse list of ids
          var _count = buf.readUInt16LE(offset + length);
          var data = [];
          length += 2;

          for (var i = 0; i < _count; i++) {
            data.push(buf.readUInt32LE(offset + length + 4 * i));
          }
          length += 4 * _count;

          bundle[id] = data;
          break;
        }

      case 8:
        {
          // Parse bundle modifications
          var _insertCount = buf.readUInt16LE(offset + length);
          length += 2;

          var insert = {};
          for (var i = 0; i < _insertCount; i++) {
            var key = buf.readUInt32LE(offset + length).toString();
            length += 4;

            // Parse zero-terminated string (key)
            var _size = 0;
            do {
              _size++;
              if (offset + length + _size > buf.length) {
                throw 'Can\'t find termination of string. Data incomplete?';
              }
            } while (buf[offset + length + _size - 1] != 0x00);

            var data = buf.slice(offset + length, offset + length + _size - 1).toString('utf8');
            insert[data] = key;
            length += _size;
          }

          var _removeCount = buf.readUInt16LE(offset + length);
          length += 2;

          var remove = [];
          for (var i = 0; i < _removeCount; i++) {
            remove.push(buf.readUInt32LE(offset + length + 4 * i).toString());
          }
          length += 4 * _removeCount;

          bundle[id] = {
            insert: insert,
            remove: remove
          };

          break;
        }

      default:
        {
          /*
           * This error doesn't hint at an invalid type, but usually indicates
           * that the header can't be properly read, due to a shifted offset.
           * This means that the data is corrupted and can't be parsed.
           */
          throw 'Unknown type ' + type + ' encountered while parsing!';
        }
    }

    offset += length;
  }

  return bundle;
}