'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = parseBinaryMap;
function parseBinaryMap(buffer) {
  var cursor = 8;
  var width = buffer.readUInt32LE(0);
  var height = buffer.readUInt32LE(4);

  function decodeExtents() {
    var res = {
      x: buffer.readFloatLE(cursor),
      y: buffer.readFloatLE(cursor + 4)
    };
    cursor += 8;
    return res;
  }

  var nw = decodeExtents();
  var ne = decodeExtents();
  var sw = decodeExtents();

  // fix for invalid map size from https://github.com/nkatsaros/pipboygo/blob/master/protocol/map.go#L55
  // originally from https://github.com/CyberShadow/csfo4/blob/master/mapfix/mapfix.d
  // thanks to nkatsaros and CyberShadow!
  if (width * height < buffer.length - 32) {
    width = (buffer.length - 32) / height;
    if (buffer.length !== 32 + width * height) {
      throw 'Invalid map stride!';
    }
  }

  return {
    width: width,
    height: height,
    pixels: buffer.slice(cursor, cursor + width * height),
    extents: {
      nw: nw,
      ne: ne,
      sw: sw
    }
  };
}